package org.beangle.webapp.security;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.struts2.ServletActionContext;
import org.apache.struts2.interceptor.NoParameters;
import org.beangle.commons.collection.ListUtil;
import org.beangle.commons.collection.MapUtil;
import org.beangle.commons.collection.SetUtil;
import org.beangle.model.query.builder.OqlBuilder;
import org.beangle.security.AuthorityException;
import org.beangle.security.Resource;
import org.beangle.security.User;
import org.beangle.security.monitor.Authentication;
import org.beangle.security.monitor.AuthenticationException;
import org.beangle.security.monitor.web.DefaultResourceExtractor;
import org.beangle.security.restriction.Param;
import org.beangle.security.restriction.Restriction;
import org.beangle.security.restriction.service.RestrictionService;
import org.beangle.security.service.AuthorityService;
import org.beangle.struts2.action.EntityDrivenAction;

import com.opensymphony.xwork2.ActionContext;

public class SecurityActionSupport extends EntityDrivenAction implements NoParameters {

	protected AuthorityService authorityService;

	protected RestrictionService restrictionService;

	// FIXME new DefaultResourceExtractor
	protected Resource getResource() {
		return authorityService.getResource(new DefaultResourceExtractor()
				.extract(ServletActionContext.getRequest()));
	}

	protected boolean isAdmin(User user) {
		return authorityService.getUserService().isAdmin(user);
	}

	protected List<Restriction> getRestrictions() {
		final Map<String, Object> session = ActionContext.getContext().getSession();
		@SuppressWarnings("unchecked")
		Map<Long, List<Restriction>> restrictionMap = (Map<Long, List<Restriction>>) session
				.get("security.restriction");
		if (null == restrictionMap) {
			restrictionMap = MapUtil.newHashMap();
			session.put("security.restriction", restrictionMap);
		}
		Resource resource = getResource();
		if (resource.getPatterns().isEmpty()) {
			return Collections.emptyList();
		}
		List<Restriction> realms = restrictionMap.get(resource.getId());
		User user = getUser();
		if (null == realms) {
			realms = restrictionService.getRestrictions(user, resource);
			restrictionMap.put(resource.getId(), realms);
		}
		// 没有权限就报错
		if (realms.isEmpty()) {
			throw new AuthorityException(resource.getName());
		}
		return realms;
	}

	protected List<Object> getRestricitonValues(String name) {
		List<Restriction> restrictions = getRestrictions();
		Set<Object> values = SetUtil.newHashSet();
		boolean gotIt = false;
		for (Restriction restiction : restrictions) {
			Param param = restiction.getParamGroup().getParam(name);
			if (null != param) {
				String value = restiction.getItem(param);
				if (null != value) {
					gotIt = true;
					values.addAll(restrictionService.select(restrictionService.getValues(param),
							restiction, param));
				}
			}
		}
		if (!gotIt) {
			List<Param> params = entityService.load(Param.class, "name", name);
			if (params.isEmpty()) {
				throw new RuntimeException("bad pattern parameter named :" + name);
			}
			Param param = (Param) params.get(0);
			return restrictionService.getValues(param);
		} else {
			return ListUtil.newArrayList(values);
		}
	}

	protected void applyRestriction(OqlBuilder<?> query) {
		Resource resource = getResource();
		restrictionService.apply(query, resource.getPatterns(), getRestrictions());
	}

	protected Long getUserId() {
		Long userId = (Long) ActionContext.getContext().getSession().get(Authentication.USERID);
		if (null == userId)
			throw new AuthenticationException();
		else
			return userId;
	}

	protected String getLoginName() {
		String loginName = (String) ActionContext.getContext().getSession().get(
				Authentication.LOGINNAME);
		if (null == loginName)
			throw new AuthenticationException();
		else
			return loginName;
	}

	protected String getFullName() {
		String fullname = (String) ActionContext.getContext().getSession().get(
				Authentication.FULLNAME);
		if (null == fullname)
			throw new AuthenticationException();
		else
			return fullname;
	}

	protected User getUser() {
		return (User) entityService.get(User.class, getUserId());
	}

	public Long getUserCategoryId() {
		return (Long) ActionContext.getContext().getSession().get(Authentication.USER_CATEGORYID);
	}

	public void setAuthorityService(AuthorityService authorityService) {
		this.authorityService = authorityService;
	}

	public void setRestrictionService(RestrictionService restrictionService) {
		this.restrictionService = restrictionService;
	}
}
