package org.beangle.webapp.security.action;

import java.util.Date;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang.StringUtils;
import org.beangle.commons.collection.MapUtil;
import org.beangle.model.query.builder.OqlBuilder;
import org.beangle.security.AdminUser;
import org.beangle.security.Group;
import org.beangle.security.Resource;
import org.beangle.security.User;
import org.beangle.security.UserCategory;
import org.beangle.security.menu.Menu;
import org.beangle.security.menu.MenuProfile;
import org.beangle.security.model.AdminUserBean;
import org.beangle.security.model.UserCategoryBean;
import org.beangle.security.restriction.Param;
import org.beangle.security.restriction.Pattern;
import org.beangle.webapp.security.SecurityActionSupport;

public class DashboardAction extends SecurityActionSupport {

	public String index() {
		return forward();
	}

	public String stat() {
		OqlBuilder<UserCategory> cquery = OqlBuilder.from(UserCategory.class, "category");
		List<UserCategory> categories = entityService.search(cquery);
		Map<Long, UserCategory> categoryMap = MapUtil.newHashMap();
		for (UserCategory category : categories) {
			categoryMap.put(category.getId(), category);
		}
		put("categories", categoryMap);

		populateUserStat();
		// stat group
		OqlBuilder<Group> groupQuery = OqlBuilder.from(Group.class, "group");
		groupQuery.select("group.category.id,group.enabled,count(*)").groupBy(
				"group.category.id,group.enabled");
		put("groupStat", entityService.search(groupQuery));

		// state menus
		List<MenuProfile> menuProfiles = entityService.loadAll(MenuProfile.class);
		Map<Long, List<?>> menuStats = MapUtil.newHashMap();
		for (MenuProfile profile : menuProfiles) {
			OqlBuilder<Menu> menuQuery = OqlBuilder.from(Menu.class, "menu");
			menuQuery.where("menu.profile=:profile", profile).select("menu.enabled,count(*)")
					.groupBy("enabled");
			menuStats.put(profile.getId(), entityService.search(menuQuery));
		}
		put("menuProfiles", menuProfiles);
		put("menuStats", menuStats);

		// stat resource
		OqlBuilder<Resource> resourceQuery = OqlBuilder.from(Resource.class, "resource");
		resourceQuery.select("resource.enabled,count(*)").groupBy("enabled");
		put("resourceStat", entityService.search(resourceQuery));

		// stat pattern and restriction
		put("patternStat", entityService.search(OqlBuilder.from(Pattern.class, "pattern").select(
				"count(*)")));
		put("paramStat", entityService.search(OqlBuilder.from(Param.class, "param").select(
				"count(*)")));
		return forward();
	}

	private void populateUserStat() {
		OqlBuilder<User> userQuery = OqlBuilder.from(User.class, "user");
		userQuery.select("user.defaultCategory.id,user.status,count(*)").groupBy(
				"user.defaultCategory.id,user.status");
		put("userStat", entityService.search(userQuery));
	}

	/**
	 * 管理用户类别和超级管理员
	 * 
	 * @return
	 */
	public String admin() {
		String newCategory = get("newCategory");
		if (StringUtils.isNotBlank(newCategory)) {
			if (entityService.load(UserCategory.class, "name", newCategory).isEmpty()) {
				UserCategoryBean category = new UserCategoryBean();
				category.setName(newCategory);
				entityService.saveOrUpdate(category);
				return redirect("admin", "info.save.success");
			} else {
				return redirect("admin", "info.save.failure");
			}
		}
		Long categoryId = getLong("removeCategoryId");
		if (null != categoryId) {
			UserCategory category = entityService.load(UserCategory.class, categoryId);
			if (null != category) {
				try{
				entityService.remove(category);
				}catch(Exception e){
					return redirect("admin", "info.remove.failure");
				}
			}
			return redirect("admin", "info.remove.success");
		}
		String newAdmin = get("newAdmin");
		if (StringUtils.isNotBlank(newAdmin)) {
			User user = authorityService.getUserService().get(newAdmin);
			if (null == user) {
				return redirect("admin", "info.save.failure");
			} else {
				if (authorityService.getUserService().isAdmin(user)) {
					return redirect("admin", "info.save.failure");
				}
				AdminUser adminUser = new AdminUserBean();
				adminUser.setUser(user);
				adminUser.setCreatedAt(new Date());
				adminUser.setUpdatedAt(new Date());
				entityService.saveOrUpdate(adminUser);
				return redirect("admin", "info.save.success");
			}
		}
		Long adminId = getLong("removeAdminId");
		if (null != adminId) {
			AdminUser adminUser = entityService.load(AdminUser.class, adminId);
			if (null != adminUser) {
				entityService.remove(adminUser);
			}
			return redirect("admin", "info.remove.success");
		}
		List<AdminUser> admins = entityService.loadAll(AdminUser.class);
		put("adminUsers", admins);
		List<UserCategory> categories = entityService.loadAll(UserCategory.class);
		put("categories", categories);
		return forward();
	}
}
