//$Id: MenuAction.java,v 1.1 2008-8-5 上午09:57:35 鄂州蚊子 Exp $
/*
 *
 * Copyright c 2005-2009
 * Licensed under the Apache License, Version 2.0 (the "License")
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 */
/********************************************************************************
 * @author 鄂州蚊子
 * 
 * MODIFICATION DESCRIPTION
 * 
 * Name                 Date                Description 
 * ============         ============        ============
 * 鄂州蚊子                 2008-8-5             Created
 *  
 ********************************************************************************/
package org.beangle.webapp.security.action;

import java.util.List;
import java.util.Set;

import org.beangle.commons.collection.ListUtil;
import org.beangle.commons.lang.SeqStringUtil;
import org.beangle.model.Entity;
import org.beangle.model.query.builder.OqlBuilder;
import org.beangle.security.Resource;
import org.beangle.security.menu.Menu;
import org.beangle.security.menu.MenuAuthority;
import org.beangle.security.menu.MenuProfile;
import org.beangle.security.menu.service.MenuService;
import org.beangle.webapp.security.SecurityActionSupport;

/**
 * 系统模块(菜单)管理响应类
 * 
 * @author 鄂州蚊子 2008-8-4
 */
public class MenuAction extends SecurityActionSupport {

	private MenuService menuService;

	protected void indexSetting() {
		put("profiles", entityService.loadAll(MenuProfile.class));
	}

	protected void editSetting(Entity<?> entity) {
		put("profiles", entityService.loadAll(MenuProfile.class));
		Menu menu = (Menu) entity;
		List<Resource> resurces = entityService.loadAll(Resource.class);
		Set<Resource> existResources = menu.getResources();
		if (null != resurces) {
			resurces.removeAll(existResources);
		}
		put("resources", resurces);
	}

	protected String saveAndForward(Entity<?> entity) {
		Menu menu = (Menu) entity;
		try {
			List<Resource> resources = ListUtil.newArrayList();
			String resourceIdSeq = get("resourceIds");
			if (null != resourceIdSeq && resourceIdSeq.length() > 0) {
				resources = entityService.load(Resource.class, "id", SeqStringUtil
						.transformToLong(resourceIdSeq));
			}
			menu.getResources().clear();
			menu.getResources().addAll(resources);
			menuService.saveOrUpdate(menu);
		} catch (Exception e) {
			return forward(ERROR);
		}
		return redirect("search", "info.save.success");
	}

	/**
	 * 禁用或激活一个或多个模块
	 * 
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return
	 */
	public String activate() {
		String menuIdSeq = get("menuIds");
		Long[] menuIds = SeqStringUtil.transformToLong(menuIdSeq);
		Boolean isActivate = getBoolean("isActivate");
		if (null != isActivate && Boolean.FALSE.equals(isActivate))
			menuService.updateState(menuIds, false);
		else
			menuService.updateState(menuIds, true);
		return redirect("search", "info.save.success");
	}

	/**
	 * 打印预览功能列表
	 * 
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return
	 * @throws Exception
	 */
	public String preview() {
		OqlBuilder<Menu> query = OqlBuilder.from(Menu.class, "menu");
		populateConditions(query);
		query.orderBy("menu.code asc");
		put("menus", entityService.search(query));

		query.cleanOrders();
		query.select("max(length(menu.code)/2)");
		List<?> rs = entityService.search(query);
		put("depth", rs.get(0));
		return forward();
	}

	@Override
	public String info() throws Exception {
		Long entityId = getEntityId(getShortName());
		if (null == entityId) {
			logger.warn("cannot get paremeter {}Id or {}.id", getShortName(), getShortName());
		}
		Entity<?> entity = getModel(getEntityName(), entityId);
		put(getShortName(), entity);
		
		OqlBuilder<MenuAuthority> groupQuery = OqlBuilder.from(MenuAuthority.class, "auth");
		groupQuery.where("auth.menu=:menu", entity).select("auth.group");
		put("groups", entityService.search(groupQuery));
		return forward();
	}

	public void setMenuService(MenuService menuService) {
		this.menuService = menuService;
	}
}
