//$Id: AuthorityAction.java,v 1.6 2006/08/28 14:40:59 chaostone Exp $
/*
 *
 * Copyright c 2005-2009
 * Licensed under the Apache License, Version 2.0 (the "License")
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 */
/********************************************************************************
 * @author chaostone
 * 
 * MODIFICATION DESCRIPTION
 * 
 * Name                 Date                Description 
 * ============         ============        ============
 * chaostone            2005-10-09          Created
 * zq                   2007-09-18          修改或替换了下面所有的info()方法
 *  
 ********************************************************************************/

package org.beangle.webapp.security.action;

import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.beangle.webapp.security.SecurityActionSupport;
import org.beangle.commons.collection.ListUtil;
import org.beangle.commons.collection.MapUtil;
import org.beangle.commons.collection.SetUtil;
import org.beangle.commons.lang.SeqStringUtil;
import org.beangle.model.entity.Model;
import org.beangle.model.query.builder.OqlBuilder;
import org.beangle.security.Authority;
import org.beangle.security.Group;
import org.beangle.security.Resource;
import org.beangle.security.User;
import org.beangle.security.UserCategory;
import org.beangle.security.monitor.AuthorityManager;
import org.beangle.security.menu.Menu;
import org.beangle.security.menu.MenuAuthority;
import org.beangle.security.menu.MenuProfile;
import org.beangle.security.menu.service.MenuAuthorityService;
import org.beangle.security.service.GroupService;
import org.beangle.struts2.convention.route.Action;

/**
 * 权限分配与管理响应类
 * 
 * @author chaostone 2005-10-9
 */
public class MenuAuthorityAction extends SecurityActionSupport {

	private GroupService groupService;

	private MenuAuthorityService menuAuthorityService;

	private AuthorityManager authorityManager;

	/**
	 * 主页面
	 */
	public String index() {
		User user = getUser();
		put("manager", user);
		if (isAdmin(user)) {
			put("allGroups", entityService.loadAll(Group.class));
		}
		return forward();
	}

	/**
	 * 根据菜单配置来分配权限
	 * 
	 * @author 鄂州蚊子
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return
	 * @throws Exception
	 */
	public String editAuthority() {
		Long groupId = getLong("group.id");
		if (null == groupId) {
			groupId = getLong("groupIds");
		}
		Group ao = groupService.get(groupId);
		User user = getUser();
		put("manager", user);
		if (isAdmin(user)) {
			put("allGroups", entityService.loadAll(Group.class));
		}
		List<UserCategory> categories = ListUtil.newArrayList();
		categories.add(((Group) ao).getCategory());

		OqlBuilder<MenuProfile> query = OqlBuilder.from(MenuProfile.class, "menuProfile");
		query.where("menuProfile.category in(:categories)", categories);
		List<MenuProfile> menuProfiles = entityService.search(query);
		put("menuProfiles", menuProfiles);

		Long menuProfileId = getLong("menuProfileId");
		MenuProfile menuProfile = null;
		if (null != menuProfileId) {
			menuProfile = entityService.get(MenuProfile.class, menuProfileId);
			if (!menuProfile.getCategory().equals(ao.getCategory())) {
				menuProfile = (menuProfiles.get(0));
			}
		} else {
			menuProfile = (menuProfiles.get(0));
		}
		if (null != menuProfile) {
			List<Menu> menus = null;
			Collection<Resource> resources = null;
			if (isAdmin(user)) {
				menus = menuProfile.getMenus();
				resources = entityService.loadAll(Resource.class);
			} else {
				menus = menuAuthorityService.getMenus(menuProfile, user);
				resources = authorityService.getResources(user);
			}
			put("resources", SetUtil.newHashSet(resources));
			boolean displayFreezen = getBool("displayFreezen");
			if (!displayFreezen) {
				List<Menu> freezed = ListUtil.newArrayList();
				for (Menu menu : menus) {
					if (!menu.isEnabled()) {
						freezed.add(menu);
					}
				}
				menus.removeAll(freezed);
			}
			put("menus", menus);

			Set<Resource> aoResources = SetUtil.newHashSet();
			Map<String, Long> aoResourceAuthorityMap = MapUtil.newHashMap();
			List<Authority> authorities = authorityService.getAuthorities(ao);
			Collection<Menu> aoMenus = menuAuthorityService.getAllMenus(menuProfile, (Group) ao);
			for (final Authority authority : authorities) {
				aoResources.add(authority.getResource());
				aoResourceAuthorityMap.put(authority.getResource().getId().toString(), authority
						.getId());
			}
			put("aoMenus", SetUtil.newHashSet(aoMenus));
			put("aoResources", aoResources);
			put("aoResourceAuthorityMap", aoResourceAuthorityMap);
		}
		put("menuProfile", menuProfile);
		put("ao", ao);
		return forward();
	}

	/**
	 * 显示权限操作提示界面
	 * 
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return
	 * @throws Exception
	 */
	public String prompt() {
		return forward();
	}

	/**
	 * 保存模块级权限
	 * 
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return
	 * @throws Exception
	 */
	public String save() {
		Group mao = groupService.get(getLong("group.id"));
		MenuProfile menuProfile = (MenuProfile) entityService.get(MenuProfile.class,
				getLong("menuProfileId"));

		Set<Menu> newMenus = SetUtil.newHashSet(entityService.load(Menu.class, "id", SeqStringUtil
				.transformToLong(get("menuId"))));

		Set<Resource> newResources = SetUtil.newHashSet(entityService.load(Resource.class, "id",
				SeqStringUtil.transformToLong(get("resourceId"))));

		// 管理员拥有的菜单权限和系统资源
		User manager = getUser();
		Set<Menu> mngMenus = null;
		Set<Resource> mngResources = SetUtil.newHashSet();
		if (isAdmin(manager)) {
			mngMenus = SetUtil.newHashSet(menuProfile.getMenus());
		} else {
			mngMenus = SetUtil.newHashSet(menuAuthorityService
					.getMenus(menuProfile, (User) manager));
		}
		for (final Menu m : mngMenus) {
			mngResources.addAll(m.getResources());
		}

		// 确定要删除的菜单和系统资源
		Set<MenuAuthority> removedMenus = SetUtil.newHashSet();
		for (MenuAuthority ma : mao.getMenuAuthorities()) {
			if (mngMenus.contains(ma.getMenu()) && ma.getMenu().getProfile().equals(menuProfile)) {
				if (!newMenus.contains(ma.getMenu())) {
					removedMenus.add(ma);
				} else {
					newMenus.remove(ma.getMenu());
				}
			}
		}

		Set<Authority> removedResources = SetUtil.newHashSet();
		for (final Authority au : mao.getAuthorities()) {
			if (mngResources.contains(au.getResource())) {
				if (!newResources.contains(au.getResource())) {
					removedResources.add(au);
				} else {
					newResources.remove(au.getResource());
				}
			}
		}

		// 删除菜单和系统资源
		mao.getMenuAuthorities().removeAll(removedMenus);
		mao.getAuthorities().removeAll(removedResources);

		// 添加新的菜单和系统资源
		for (Menu menu : newMenus) {
			MenuAuthority authority = Model.newInstance(MenuAuthority.class);
			authority.setGroup(mao);
			authority.setMenu(menu);
			mao.getMenuAuthorities().add(authority);
		}

		for (Resource resource : newResources) {
			Authority authority = Model.newInstance(Authority.class);
			authority.setGroup(mao);
			authority.setResource(resource);
			mao.getAuthorities().add(authority);
		}

		entityService.saveOrUpdate(mao);
		authorityManager.registerGroupAuthorities(mao);

		Action redirect = Action.to(this).method("editAuthority");
		redirect.param("group.id", mao.getId()).param("menuProfileId", menuProfile.getId());
		String displayFreezen = get("displayFreezen");
		if (null != displayFreezen) {
			redirect.param("displayFreezen", displayFreezen);
		}
		return redirect(redirect, "info.save.success");
	}

	public void setGroupService(GroupService groupService) {
		this.groupService = groupService;
	}

	public void setMenuAuthorityService(MenuAuthorityService menuAuthorityService) {
		this.menuAuthorityService = menuAuthorityService;
	}

	public void setAuthorityManager(AuthorityManager authorityManager) {
		this.authorityManager = authorityManager;
	}
}
