/*
 *
 * Copyright c 2005-2009
 * Licensed under the Apache License, Version 2.0 (the "License")
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 */
/********************************************************************************
 * @author chaostone
 * 
 * MODIFICATION DESCRIPTION
 * 
 * Name                 Date                Description 
 * ============         ============        ============
 * chaostone             2006-5-24            Created
 *  
 ********************************************************************************/
package org.beangle.webapp.security.action;

import java.util.Collections;
import java.util.Date;
import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.apache.struts2.ServletActionContext;
import org.beangle.commons.collection.ListUtil;
import org.beangle.commons.collection.page.PagedList;
import org.beangle.commons.comparators.PropertyComparator;
import org.beangle.security.monitor.OnlineActivity;
import org.beangle.security.monitor.auth.session.CategoryProfile;
import org.beangle.security.monitor.auth.session.service.ProfileSessionController;
import org.beangle.security.monitor.web.Accesslog;
import org.beangle.security.monitor.web.CachedResourceAccessor;
import org.beangle.webapp.security.SecurityActionSupport;

import com.opensymphony.xwork2.ActionContext;

/**
 * 系统在线用户管理
 * 
 * @author chaostone
 * 
 */
public class MonitorAction extends SecurityActionSupport {

	private ProfileSessionController sessionController;

	public String profiles() {
		put("onlineProfiles", sessionController.getOnlineProfiles());
		return forward();
	}

	public String activities() {
		String orderBy = get("orderBy");
		if (StringUtils.isEmpty(orderBy)) {
			orderBy = "principal asc";
		}
		List<OnlineActivity> onlineActivities = sessionController.getOnlineActivities();
		Collections.sort(onlineActivities, new PropertyComparator<Object>(orderBy));
		put("onlineActivities", new PagedList<OnlineActivity>(onlineActivities, getPageLimit()));
		put("onlineProfiles", sessionController.getOnlineProfiles());
		put("now",new Date());
		return forward();
	}

	/**
	 * 保存设置
	 * 
	 */
	public String saveProfile() {
		List<CategoryProfile> categories = entityService.loadAll(CategoryProfile.class);
		for (final CategoryProfile profile : categories) {
			int max = getInteger("max_" + profile.getId()).intValue();
			int maxSessions = getInteger("maxSessions_" + profile.getId()).intValue();
			int inactiveInterval = getInteger("inactiveInterval_" + profile.getId()).intValue();
			profile.setCapacity(max);
			profile.setUserMaxSessions(maxSessions);
			profile.setInactiveInterval(inactiveInterval);
		}
		entityService.saveOrUpdate(categories);
		sessionController.loadProfiles();
		return redirect("profiles", "info.save.success");
	}

	public String invalidate() {
		String[] sessionIds = (String[])getAll("sessionId");
		String mySessionId = ServletActionContext.getRequest().getSession().getId();
		if (null != sessionIds) {
			for (int i = 0; i < sessionIds.length; i++) {
				if (mySessionId.equals(sessionIds[i]))
					continue;
				sessionController.removeAuthentication(sessionIds[i]);
			}
		}
		return redirect("activities", "info.action.success");
	}

	/**
	 * 访问记录
	 */
	public String accesslogs() {
		CachedResourceAccessor ra = (CachedResourceAccessor) ActionContext.getContext().getApplication()
				.get("ResourceAccessor");
		List<Accesslog> accessLogs = null;
		if (null == ra) {
			accessLogs = Collections.emptyList();
		} else {
			accessLogs = ListUtil.newArrayList(ra.getAccessLogs());
		}
		String orderBy = get("orderBy");
		if (StringUtils.isEmpty(orderBy)) {
			orderBy = "duration desc";
		}
		Collections.sort(accessLogs, new PropertyComparator<Object>(orderBy));
		put("accesslogs",new PagedList<Accesslog>(accessLogs, getPageLimit()));
		return forward();
	}

	public void setSessionController(ProfileSessionController sessionController) {
		this.sessionController = sessionController;
	}

}
