//$Id: ResourceAction.java,v 1.7 2006/10/20 10:43:19 chaostone Exp $
/*
 *
 * Copyright c 2005-2009
 * Licensed under the Apache License, Version 2.0 (the "License")
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 */
/********************************************************************************
 * @author chaostone
 * 
 * MODIFICATION DESCRIPTION
 * 
 * Name                 Date                Description 
 * ============         ============        ============
 * chaostone             2005-10-9         Created
 *  
 ********************************************************************************/

package org.beangle.webapp.security.action;

import java.util.Collections;
import java.util.List;

import org.beangle.commons.lang.SeqStringUtil;
import org.beangle.model.Entity;
import org.beangle.model.query.builder.OqlBuilder;
import org.beangle.security.Authority;
import org.beangle.security.Resource;
import org.beangle.security.UserCategory;
import org.beangle.security.menu.Menu;
import org.beangle.security.monitor.AuthorityManager;
import org.beangle.security.restriction.Pattern;
import org.beangle.security.service.ResourceService;
import org.beangle.struts2.convention.route.Action;
import org.beangle.webapp.security.SecurityActionSupport;

/**
 * 系统模块管理响应类
 * 
 * @author chaostone 2005-10-9
 */
public class ResourceAction extends SecurityActionSupport {

	private ResourceService resourceService;

	private AuthorityManager authorityManager;

	/**
	 * 禁用或激活一个或多个模块
	 * 
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return
	 */
	public String activate() {
		Long[] resourceIds = SeqStringUtil.transformToLong(get("resourceIds"));
		Boolean enabled = getBoolean("enabled");
		if (null == enabled) {
			enabled = Boolean.FALSE;
		}
		resourceService.updateState(resourceIds, enabled.booleanValue());
		authorityManager.refreshCache();
		return redirect("search", "info.save.success");
	}

	@Override
	public String index() throws Exception {
		return forward(new Action(this, "search"));
	}

	protected void editSetting(Entity<?> entity) {
		Resource resource = (Resource) entity;
		List<Pattern> patterns = entityService.loadAll(Pattern.class);
		patterns.removeAll(resource.getPatterns());
		put("patterns", patterns);
		put("categories", entityService.loadAll(UserCategory.class));
	}

	protected String saveAndForward(Entity<?> entity) {
		Resource resource = (Resource) entity;
		if (null != resource) {
			List<?> list = this.entityService.load(Resource.class, "name", resource.getName());
			int isUniqueFlag = 0;
			if (null != resource.getId()) {
				isUniqueFlag = 1;
			}
			if (null != list && list.size() > isUniqueFlag) {
				return redirect("edit", "error.notUnique");
			}
		}

		Long[] patternIds = SeqStringUtil.transformToLong(get("patternIds"));
		List<Pattern> patterns = Collections.emptyList();
		if (null != patternIds) {
			patterns = entityService.load(Pattern.class, "id", patternIds);
		}
		resource.getPatterns().clear();
		resource.getPatterns().addAll(patterns);

		String categoryIds = get("categoryIds");
		List<UserCategory> categories = entityService.load(UserCategory.class, "id", SeqStringUtil
				.transformToLong(categoryIds));
		resource.getCategories().clear();
		resource.getCategories().addAll(categories);

		entityService.saveOrUpdate(resource);
		authorityManager.refreshCache();

		logger.info("save resource success {}", resource.getTitle());
		return redirect("search", "info.save.success");
	}

	public String info() {
		Long entityId = getEntityId(getShortName());
		Entity<?> entity = getModel(entityName, entityId);
		OqlBuilder<Menu> query = OqlBuilder.from(Menu.class, "menu");
		query.join("menu.resources", "r").where("r.id=:resourceId", entity.getEntityId()).orderBy(
				"menu.profile.id,menu.code");
		
		OqlBuilder<Authority> groupQuery = OqlBuilder.from(Authority.class, "auth");
		groupQuery.where("auth.resource=:resource", entity).select("auth.group");

		OqlBuilder<?> userQuery = OqlBuilder.hql("select distinct u from User u join u.groups g,Authority a where g=a.group and a.resource=:resource");
		userQuery.param("resource", entity);
		
		put(getShortName(), entity);
		put("users",entityService.search(userQuery));
		put("groups", entityService.search(groupQuery));
		put("menus", entityService.search(query));
		put("categories", entityService.loadAll(UserCategory.class));
		return forward();
	}

	public void setResourceService(ResourceService resourceService) {
		this.resourceService = resourceService;
	}

	public void setAuthorityManager(AuthorityManager authorityManager) {
		this.authorityManager = authorityManager;
	}

}
