//$Id: RestrictionAction.java,v 1.1 2008-8-8 下午01:54:09 鄂州蚊子 Exp $
/*
 *
 * Copyright c 2005-2009
 * Licensed under the Apache License, Version 2.0 (the "License")
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 */
/********************************************************************************
 * @author 鄂州蚊子
 * 
 * MODIFICATION DESCRIPTION
 * 
 * Name                 Date                Description 
 * ============         ============        ============
 * 鄂州蚊子                 2008-8-8             Created
 *  
 ********************************************************************************/
package org.beangle.webapp.security.action;

import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.lang.StringUtils;
import org.beangle.commons.collection.ListUtil;
import org.beangle.commons.collection.MapUtil;
import org.beangle.commons.collection.SetUtil;
import org.beangle.model.entity.Model;
import org.beangle.security.Authority;
import org.beangle.security.User;
import org.beangle.security.restriction.Param;
import org.beangle.security.restriction.ParamGroup;
import org.beangle.security.restriction.Restriction;
import org.beangle.security.restriction.RestrictionHolder;
import org.beangle.struts2.helper.Params;
import org.beangle.webapp.security.SecurityActionSupport;
import org.beangle.webapp.security.helper.RestrictionHelper;

public class RestrictionAction extends SecurityActionSupport {

	public String tip() {
		return forward();
	}

	/**
	 * 删除数据限制权限
	 */
	public String remove() {
		Restriction restriction = getRestriction();
		RestrictionHolder holer = new RestrictionHelper(entityService).getHolder();
		holer.getRestrictions().remove(restriction);
		entityService.saveOrUpdate(holer);
		return redirect("info", "info.delete.success");
	}

	/**
	 * 查看限制资源界面
	 */
	public String info() {
		RestrictionHelper helper = new RestrictionHelper(entityService);
		helper.setRestrictionService(restrictionService);
		helper.populateInfo(helper.getHolder());
		return forward();
	}

	public String save() {
		Restriction restriction = getRestriction();
		RestrictionHolder holder = new RestrictionHelper(entityService).getHolder();
		List<Restriction> myRestrictions = getMyRestrictions(restriction.getParamGroup(), holder);
		Set<Param> ignoreParams = getIgnoreParams(myRestrictions);
		boolean isAdmin = isAdmin(getUser());
		for (Param param : restriction.getParamGroup().getParams()) {
			String value = get(param.getName());
			if ((ignoreParams.contains(param) || isAdmin) && getBool("ignoreParam" + param.getId())) {
				restriction.setItem(param, "*");
			} else {
				if (StringUtils.isEmpty(value)) {
					restriction.getItems().remove(param.getId());
				} else {
					restriction.setItem(param, value);
				}
			}
		}
		if (restriction.getItems().isEmpty()) {
			holder.getRestrictions().remove(restriction);
			entityService.saveOrUpdate(holder);
			return redirect("info", "info.save.success");
		} else {
			if (restriction.unsaved()) {
				holder.getRestrictions().add(restriction);
				entityService.saveOrUpdate(holder);
			} else {
				entityService.saveOrUpdate((String) RestrictionHelper.restrictionTypeMap
						.get(get("restrictionType")), restriction);
			}
			return redirect("info", "info.save.success");
		}
	}

	/**
	 * 编辑权限<br>
	 */
	public String edit() {
		// 取得各参数的值
		Restriction restriction = getRestriction();
		boolean isAdmin = isAdmin(getUser());
		Map<String, Object> mngParams = MapUtil.newHashMap();
		Map<String, Object> aoParams = MapUtil.newHashMap();
		List<Restriction> myRestricitons = getMyRestrictions(restriction.getParamGroup(),
				restriction.getHolder());
		Set<Param> ignores = getIgnoreParams(myRestricitons);
		put("ignoreParams", ignores);
		Set<Param> holderIgnoreParams = SetUtil.newHashSet();
		put("holderIgnoreParams", holderIgnoreParams);
		for (Param param : restriction.getParamGroup().getParams()) {
			List<Object> mngParam = restrictionService.getValues(param);
			if (!isAdmin) {
				mngParam.retainAll(getMyRestrictionValues(myRestricitons, param.getName()));
			} else {
				ignores.add(param);
			}
			String paramValue = restriction.getItem(param);
			if ("*".equals(paramValue)) {
				holderIgnoreParams.add(param);
			}
			mngParams.put(param.getName(), mngParam);
			if (null == param.getEditor()) {
				aoParams.put(param.getName(), paramValue);
			} else {
				Set<Object> aoParam = restrictionService.select(
						restrictionService.getValues(param), restriction, param);
				aoParams.put(param.getName(), aoParam);
			}
		}
		put("mngParams", mngParams);
		put("aoParams", aoParams);
		put("restriction", restriction);
		return forward();
	}

	private List<Restriction> getMyRestrictions(ParamGroup paramGroup, RestrictionHolder holder) {
		String type = get("restrictionType");
		List<Restriction> restrictions = ListUtil.newArrayList();
		User me = getUser();
		if (type.equals("user")) {
			restrictions = ListUtil.newArrayList(((RestrictionHolder) me).getRestrictions());
		} else if (type.equals("group")) {
			for (RestrictionHolder group : me.getGroups()) {
				restrictions.addAll(group.getRestrictions());
			}
		} else if (type.equals("authority")) {
			restrictions = restrictionService.getAuthorityRestrictions(me, ((Authority) holder)
					.getResource());
		}
		List<Restriction> rt = ListUtil.newArrayList();
		for (Restriction restriction : restrictions) {
			if (!restriction.getParamGroup().equals(paramGroup)) {
				continue;
			}
			rt.add(restriction);
		}
		return rt;
	}

	private Set<Param> getIgnoreParams(List<Restriction> restrictions) {
		Set<Param> ignores = SetUtil.newHashSet();
		for (Restriction restriction : restrictions) {
			for (Param param : restriction.getParamGroup().getParams()) {
				String value = restriction.getItem(param);
				if ("*".equals(value)) {
					ignores.add(param);
				}
			}
		}
		return ignores;
	}

	private List<Object> getMyRestrictionValues(List<Restriction> restrictions, String name) {
		List<Object> values = ListUtil.newArrayList();
		for (Restriction restriction : restrictions) {
			Param param = restriction.getParamGroup().getParam(name);
			if (null != param) {
				String value = restriction.getItem(param);
				if (null != value) {
					values.addAll(restrictionService.select(restrictionService.getValues(param),
							restriction, param));
				}
			}
		}
		return values;
	}

	private Restriction getRestriction() {
		Long restrictionId = getLong("restriction.id");
		Restriction restriction = null;
		String entityName = (String) RestrictionHelper.restrictionTypeMap
				.get(get("restrictionType"));
		if (null == restrictionId) {
			restriction = (Restriction) Model.getEntityType(entityName).newInstance();
		} else {
			restriction = (Restriction) entityService.get(entityName, restrictionId);
		}
		populate(Params.sub("restriction"), restriction, entityName);
		if (null == restrictionId) {
			restriction.setParamGroup((ParamGroup) entityService.get(ParamGroup.class, restriction
					.getParamGroup().getId()));
		}
		return restriction;
	}
}
