//$Id: UserAction.java,v 1.19 2007/01/24 01:11:32 chaostone Exp $
/*
 *
 * Copyright c 2005-2009
 * Licensed under the Apache License, Version 2.0 (the "License")
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 */
/********************************************************************************
 * @author chaostone
 * 
 * MODIFICATION DESCRIPTION
 * 
 * Name                 Date                Description 
 * ============         ============        ============
 * dell                                     Created
 * chaostone            2005-09-29          refactor
 * zq                   2007-09-18          修改或替换了下面所有的info()方法
 *  
 ********************************************************************************/

package org.beangle.webapp.security.action;

import java.util.Collection;
import java.util.List;
import java.util.Set;

import org.apache.commons.lang.StringUtils;
import org.beangle.commons.collection.ListUtil;
import org.beangle.commons.collection.Order;
import org.beangle.commons.collection.SetUtil;
import org.beangle.commons.lang.SeqStringUtil;
import org.beangle.model.Entity;
import org.beangle.model.query.builder.Condition;
import org.beangle.model.query.builder.OqlBuilder;
import org.beangle.model.transfer.exporter.PropertyExtractor;
import org.beangle.security.Group;
import org.beangle.security.User;
import org.beangle.security.UserCategory;
import org.beangle.security.codec.EncryptUtil;
import org.beangle.security.service.GroupService;
import org.beangle.security.service.UserService;
import org.beangle.struts2.action.ActionTextResource;
import org.beangle.struts2.convention.route.Action;
import org.beangle.webapp.security.SecurityActionSupport;
import org.beangle.webapp.security.helper.UserPropertyExtractor;

/**
 * 用户管理响应处理类
 * 
 * @author chaostone 2005-9-29
 */
public class UserAction extends SecurityActionSupport {

	private UserService userService;

	private GroupService groupService;

	protected void indexSetting() {
		put("categories", entityService.loadAll(UserCategory.class));
	}

	protected OqlBuilder<User> getQueryBuilder() {
		User manager = getUser();
		OqlBuilder<User> userQuery = OqlBuilder.from(entityName, "user");

		// 查询用户组
		StringBuilder sb = new StringBuilder("exists(from user.groups ug where ");
		List<Object> params = ListUtil.newArrayList();
		boolean queryGroup=false;
		if (!userService.isAdmin(manager)) {
			if (manager.getMngGroups().isEmpty()) {
				sb.append("1=0");
			} else {
				sb.append("ug in(:groups) ");
				params.add(manager.getMngGroups());
			}
			queryGroup=true;
		}
		String groupName = get("groupName");
		if (StringUtils.isNotEmpty(groupName)) {
			if (queryGroup) {
				sb.append(" and ");
			}
			sb.append("ug.name like :groupName ");
			params.add("%" + groupName + "%");
			queryGroup=true;
		}
		if (queryGroup) {
			sb.append(')');
			Condition groupCondition = new Condition(sb.toString());
			groupCondition.params(params);
			userQuery.where(groupCondition);
		}

		Long categoryId = getLong("categoryId");
		if (null != categoryId) {
			userQuery.join("user.categories", "category");
			userQuery.where("category.id=:categoryId", categoryId);
		}
		populateConditions(userQuery);
		userQuery.orderBy(get(Order.ORDER_STR)).limit(getPageLimit());
		return userQuery;
	}

	protected PropertyExtractor getPropertyExtractor() {
		return new UserPropertyExtractor(new ActionTextResource(this));
	}

	/**
	 * 保存用户信息
	 */
	protected String saveAndForward(Entity<?> entity) {
		User user = (User) entity;
		String errorMsg = "";
		user.getCategories().clear();
		// // 收集用户身份
		String[] categories = StringUtils.split(get("categoryIds"), ",");
		for (int i = 0; i < categories.length; i++) {
			errorMsg = checkCategory(user, Long.valueOf(categories[i]));
			if (StringUtils.isNotEmpty(errorMsg)) {
				return forward(new Action("edit"), errorMsg);
			}
		}
		// 检验用户合法性
		errorMsg = checkUser(user);
		if (StringUtils.isNotEmpty(errorMsg)) {
			return forward(new Action("edit"), errorMsg);
		}
		try {
			String groupIdSeq = get("groupIds");
			String mngGroupIdSeq = get("mngGroupIds");
			List<Group> groups = groupService.get(SeqStringUtil.transformToLong(groupIdSeq));
			List<Group> mngGroups = groupService.get(SeqStringUtil.transformToLong(mngGroupIdSeq));
			user.getGroups().clear();
			user.getGroups().addAll(groups);
			user.getMngGroups().clear();
			user.getMngGroups().addAll(mngGroups);
			processPassword(user);
			if (user.unsaved()) {
				User creator = userService.get(getUserId());
				userService.createUser(creator, user);
			} else {
				userService.saveOrUpdate(user);
			}
		} catch (Exception e) {
			// FIXME
			return forward(ERROR);
		}
		return redirect("search", "info.save.success");
	}

	protected void editSetting(Entity<?> entity) {
		User user = (User) entity;
		User manager = getUser();
		Collection<Group> allGroups = manager.getMngGroups();
		if (userService.isAdmin(manager)) {
			allGroups = entityService.loadAll(Group.class);
		}
		Set<Group> userGroups = SetUtil.newHashSet(allGroups);
		userGroups.removeAll(user.getGroups());
		Set<Group> mngGroups = SetUtil.newHashSet(allGroups);
		mngGroups.removeAll(user.getMngGroups());
		put("userGroups", userGroups);
		put("mngGroups", mngGroups);
		put("categories", entityService.loadAll(UserCategory.class));
	}

	/**
	 * 删除一个或多个用户
	 * 
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return
	 */
	public String remove() {
		String userIdSeq = get("userIds");
		Long[] userIds = SeqStringUtil.transformToLong(userIdSeq);
		User creator = userService.get(getUserId());
		List<User> toBeRemoved = userService.getUsers(userIds);
		try {
			for (User one : toBeRemoved) {
				// 不能删除自己
				if (!one.getId().equals(getUserId())) {
					userService.removeUser(creator, one);
				}
			}
		} catch (Exception e) {
			return redirect("search", "info.delete.failure");
		}
		return redirect("search", "info.delete.success");
	}

	/**
	 * 禁用或激活一个或多个用户
	 * 
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return
	 */
	public String activate() {
		String userIdSeq = get("userIds");
		Long[] userIds = SeqStringUtil.transformToLong(userIdSeq);
		String isActivate = get("isActivate");
		try {
			if (StringUtils.isNotEmpty(isActivate) && "false".equals(isActivate)) {
				// logHelper.info(request, "UnActivate userIds:" + userIds);
				userService.updateState(userIds, User.FREEZE);
			} else {
				// logHelper.info(request, "Activate userIds:" + userIds);
				userService.updateState(userIds, User.ACTIVE);
			}
		} catch (Exception e) {
			// logHelper.info(request, "Faliure Activate alert for userIds:"
			// + userIds, e);
			return forward(ERROR, "error.occurred");
		}
		String msg = "ok.activate";
		if (StringUtils.isNotEmpty(isActivate) && "false".equals(isActivate))
			msg = "info.unactivate.success";

		return redirect("search", msg);
	}

	/**
	 * 核实用户身份
	 * 
	 * @param user
	 * @param category
	 * @return
	 */
	protected String checkCategory(User user, Long categoryId) {
		user.getCategories().add(entityService.get(UserCategory.class, categoryId));
		return "";
	}

	protected String checkUser(User user) {
		if (user.unsaved() && entityService.exist(entityName, "name", user.getName())) {
			return "error.model.existed";
		}
		return "";
	}

	public String info() throws Exception {
		String loginName = get("loginName");
		if (StringUtils.isNotBlank(loginName)) {
			User user = userService.get(loginName);
			if (null != user) {
				put("user", user);
				return forward();
			} else {
				return null;
			}
		} else {
			return super.info();
		}
	}

	protected void processPassword(User user) {
		String password = get("password");
		if (StringUtils.isNotBlank(password)) {
			user.setPassword(EncryptUtil.encode(password));
		} else if (user.unsaved()) {
			password = User.DEFAULT_PASSWORD;
			user.setPassword(EncryptUtil.encode(password));
		}
	}

	public void setUserService(UserService userService) {
		this.userService = userService;
	}

	public void setGroupService(GroupService groupService) {
		this.groupService = groupService;
	}

}
