package org.beangle.webapp.security.action;

import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.beangle.commons.collection.MapUtil;
import org.beangle.commons.collection.SetUtil;
import org.beangle.commons.collection.page.PageLimit;
import org.beangle.model.query.builder.OqlBuilder;
import org.beangle.security.Group;
import org.beangle.security.Resource;
import org.beangle.security.User;
import org.beangle.security.menu.Menu;
import org.beangle.security.menu.MenuProfile;
import org.beangle.security.menu.service.MenuAuthorityService;
import org.beangle.security.monitor.OnlineActivity;
import org.beangle.security.monitor.SecurityManager;
import org.beangle.security.monitor.auth.session.SessionActivity;
import org.beangle.security.restriction.RestrictionHolder;
import org.beangle.security.service.UserService;
import org.beangle.struts2.convention.route.Action;
import org.beangle.webapp.security.SecurityActionSupport;
import org.beangle.webapp.security.helper.RestrictionHelper;

public class UserDashboardAction extends SecurityActionSupport {

	private MenuAuthorityService menuAuthorityService;

	private SecurityManager securityManager;

	private UserService userService;

	public String index() {
		Long userId = getLong("user.id");
		User me = getUser();
		if (null != userId) {
			User managed = (User) entityService.get(User.class, userId);
			if (me.equals(managed) || userService.isManagedBy(me, managed)) {
				buildDashboard(managed);
				return forward();
			} else {
				return forward(ERROR);
			}

		} else {
			buildDashboard(me);
		}
		return forward();
	}

	private void buildDashboard(User user) {
		put("user", user);
		populateMenus(user);
		populateSessionActivities(user);
		populateOnlineActivities(user);
		RestrictionHelper helper = new RestrictionHelper(entityService);
		helper.setRestrictionService(restrictionService);
		helper.populateInfo((RestrictionHolder) user);
	}

	private void populateOnlineActivities(User user) {
		Collection<OnlineActivity> onlineActivities = securityManager.getAuthenticationManager()
				.getSessionController().getOnlineActivities(user);
		put("onlineActivities", onlineActivities);
	}

	private void populateSessionActivities(User user) {
		OqlBuilder<SessionActivity> onlineQuery = OqlBuilder.from(SessionActivity.class,
				"sessionActivity");
		onlineQuery.where("sessionActivity.name =:name", user.getName());
		PageLimit limit = getPageLimit();
		limit.setPageSize(10);
		onlineQuery.orderBy("sessionActivity.loginAt desc").limit(limit);
		put("sessionActivities", entityService.search(onlineQuery));
	}

	private void populateMenus(User user) {
		OqlBuilder<MenuProfile> query = OqlBuilder.from(MenuProfile.class, "menuProfile");
		query.where("menuProfile.category in(:categories)", user.getCategories());
		List<?> menuProfiles = entityService.search(query);
		put("menuProfiles", menuProfiles);

		Long menuProfileId = getLong("menuProfileId");
		if (null == menuProfileId && !menuProfiles.isEmpty()) {
			menuProfileId = ((MenuProfile) (menuProfiles.get(0))).getId();
		}
		if (null != menuProfileId) {
			MenuProfile menuProfile = (MenuProfile) entityService.get(MenuProfile.class,
					menuProfileId);
			List<Menu> menus = menuAuthorityService.getMenus(menuProfile, user);
			Set<Resource> resources = SetUtil.newHashSet(authorityService.getResources(user));
			Map<String, Group> groupMap = MapUtil.newHashMap();
			Map<String, List<Menu>> groupMenusMap = MapUtil.newHashMap();

			for (Group group : user.getGroups()) {
				groupMap.put(group.getId().toString(), group);
				groupMenusMap.put(group.getId().toString(), menuAuthorityService.getMenus(
						menuProfile, group));
			}
			put("menus", menus);
			put("groupMap", groupMap);
			put("groupMenusMap", groupMenusMap);
			put("resources", resources);
		}
	}

	public String restrictionInfo() {
		User user = getUser();
		return forward(new Action(RestrictionAction.class, "info", "&restriction.holder.id="
				+ user.getId() + "&restrictionType=user"));
	}

	public void setMenuAuthorityService(MenuAuthorityService menuAuthorityService) {
		this.menuAuthorityService = menuAuthorityService;
	}

	public void setSecurityManager(SecurityManager securityManager) {
		this.securityManager = securityManager;
	}

	public void setUserService(UserService userService) {
		this.userService = userService;
	}

}
