package org.beangle.webapp.security.helper;

import java.util.Collections;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang.StringUtils;
import org.beangle.commons.collection.ListUtil;
import org.beangle.commons.collection.MapUtil;
import org.beangle.commons.comparators.PropertyComparator;
import org.beangle.model.persist.EntityService;
import org.beangle.security.Authority;
import org.beangle.security.Group;
import org.beangle.security.User;
import org.beangle.security.restriction.Param;
import org.beangle.security.restriction.ParamGroup;
import org.beangle.security.restriction.Pattern;
import org.beangle.security.restriction.Restriction;
import org.beangle.security.restriction.RestrictionHolder;
import org.beangle.security.restriction.service.RestrictionService;
import org.beangle.struts2.helper.ContextHelper;
import org.beangle.struts2.helper.Params;

public class RestrictionHelper {

	public static final Map<String, String> restrictionTypeMap = MapUtil.newHashMap();
	static {
		restrictionTypeMap.put("user", "org.beangle.security.restriction.UserRestriction");
		restrictionTypeMap.put("group", "org.beangle.security.restriction.GroupRestriction");
		restrictionTypeMap
				.put("authority", "org.beangle.security.restriction.AuthorityRestriction");
	}

	EntityService entityService;

	RestrictionService restrictionService;

	
	public RestrictionHelper(EntityService entityService) {
		super();
		this.entityService = entityService;
	}

	public RestrictionHolder getHolder() {
		Long restrictionHolderId = Params.getLong("restriction.holder.id");
		String restrictionType = Params.get("restrictionType");
		RestrictionHolder holer = null;
		if ("user".equals(restrictionType)) {
			holer = (RestrictionHolder) entityService.get(User.class, restrictionHolderId);
		} else if ("group".equals(restrictionType)) {
			holer = (RestrictionHolder) entityService.get(Group.class, restrictionHolderId);
		} else {
			holer = (RestrictionHolder) entityService.get(Authority.class, restrictionHolderId);
		}
		return holer;
	}

	/**
	 * 查看限制资源界面
	 */
	public void populateInfo(RestrictionHolder holder) {
		List<Restriction> restrictions = ListUtil.newArrayList(holder.getRestrictions());
		Collections.sort(restrictions, new PropertyComparator<Object>("paramGroup.name"));
		Map<String, Map<String, Object>> paramMaps = MapUtil.newHashMap();
		for (final Restriction restriction : restrictions) {
			Map<String, Object> aoParams = MapUtil.newHashMap();
			for (Param param : restriction.getParamGroup().getParams()) {
				String value = restriction.getItem(param);
				if (StringUtils.isNotEmpty(value)) {
					if (null == param.getEditor()) {
						aoParams.put(param.getName(), value);
					} else {
						aoParams.put(param.getName(), restrictionService.select(restrictionService
								.getValues(param), restriction, param));
					}
				}
			}
			paramMaps.put(restriction.getId().toString(), aoParams);
		}
		String forEdit = Params.get("forEdit");
		if (StringUtils.isNotEmpty(forEdit)) {
			List<ParamGroup> paramGroups = ListUtil.newArrayList();
			if (holder instanceof Authority) {
				Authority au = (Authority) holder;
				for (Pattern pattern : au.getResource().getPatterns()) {
					paramGroups.add(pattern.getParamGroup());
				}
			} else {
				paramGroups = entityService.loadAll(ParamGroup.class);
			}
			ContextHelper.put("paramGroups", paramGroups);
		}
		ContextHelper.put("paramMaps", paramMaps);
		ContextHelper.put("restrictions", restrictions);
	}

	public RestrictionService getRestrictionService() {
		return restrictionService;
	}

	public void setRestrictionService(RestrictionService restrictionService) {
		this.restrictionService = restrictionService;
	}

}
